#include "button_handler.h"
#include "config.h"

ButtonHandler::ButtonHandler(Adafruit_MCP23X17* mcp1, Adafruit_MCP23X17* mcp2) {
  this->mcp1 = mcp1;
  this->mcp2 = mcp2;
  
  // Initialisiere alle Button-Zustände
  for(int i = 0; i < NUM_BUTTONS; i++) {
    buttonStates[i] = false;
    lastButtonStates[i] = false;
    lastDebounceTime[i] = 0;
  }
}

void ButtonHandler::init() {
  // Konfiguriere alle Button-Pins als Eingänge mit Pull-Up
  // MCP1: Buttons 0-15 (Port A und B)
  for(int i = 0; i < 16; i++) {
    mcp1->pinMode(i, INPUT_PULLUP);
  }
  
  // MCP2: Buttons 16-19 (Port A, Pins 0-3)
  for(int i = 0; i < 4; i++) {
    mcp2->pinMode(i, INPUT_PULLUP);
  }
  
  Serial.println("Button Handler initialisiert - 20 Buttons konfiguriert");
}

void ButtonHandler::update() {
  unsigned long currentTime = millis();
  
  // Lese alle Button-Zustände
  for(int i = 0; i < NUM_BUTTONS; i++) {
    bool currentState;
    
    // Bestimme welcher MCP und welcher Pin
    if(i < 16) {
      // Buttons 0-15 auf MCP1
      currentState = !mcp1->digitalRead(i); // Invertiert wegen Pull-Up
    } else {
      // Buttons 16-19 auf MCP2
      currentState = !mcp2->digitalRead(i - 16); // Invertiert wegen Pull-Up
    }
    
    // Debouncing: Prüfe ob genug Zeit vergangen ist
    if(currentTime - lastDebounceTime[i] > DEBOUNCE_TIME) {
      if(currentState != lastButtonStates[i]) {
        // Zustand hat sich geändert - speichere neuen Zustand
        lastButtonStates[i] = buttonStates[i];
        buttonStates[i] = currentState;
        lastDebounceTime[i] = currentTime;
      }
    }
  }
}

bool ButtonHandler::isPressed(uint8_t buttonIndex) {
  if(buttonIndex >= NUM_BUTTONS) return false;
  return buttonStates[buttonIndex];
}

bool ButtonHandler::wasPressed(uint8_t buttonIndex) {
  if(buttonIndex >= NUM_BUTTONS) return false;
  // Button wurde gedrückt wenn er jetzt gedrückt ist, aber vorher nicht
  return buttonStates[buttonIndex] && !lastButtonStates[buttonIndex];
}

bool ButtonHandler::wasReleased(uint8_t buttonIndex) {
  if(buttonIndex >= NUM_BUTTONS) return false;
  // Button wurde losgelassen wenn er jetzt nicht gedrückt ist, aber vorher schon
  return !buttonStates[buttonIndex] && lastButtonStates[buttonIndex];
}
