#include "multiplexer.h"

Multiplexer::Multiplexer(uint8_t s0Pin, uint8_t s1Pin, uint8_t s2Pin, uint8_t s3Pin, uint8_t analogPin) {
  this->s0 = s0Pin;
  this->s1 = s1Pin;
  this->s2 = s2Pin;
  this->s3 = s3Pin;
  this->analogPin = analogPin;
  
  // Initialisiere alle Werte mit -1 um erste Änderung zu erkennen
  for(int i = 0; i < 16; i++) {
    lastValues[i] = -1;
    lastReadTime[i] = 0;
  }
}

void Multiplexer::init() {
  // Konfiguriere Multiplexer-Steuerpins als Ausgänge
  pinMode(s0, OUTPUT);
  pinMode(s1, OUTPUT);
  pinMode(s2, OUTPUT);
  pinMode(s3, OUTPUT);
  
  // Konfiguriere analogen Eingang
  pinMode(analogPin, INPUT);
  
  Serial.println("CD74HC4067 Multiplexer initialisiert");
}

void Multiplexer::selectChannel(uint8_t channel) {
  // Stelle sicher dass der Kanal im gültigen Bereich ist
  if(channel > 15) return;
  
  // Setze die Multiplexer-Steuerleitungen entsprechend dem Kanal
  digitalWrite(s0, (channel & 0x01) ? HIGH : LOW);
  digitalWrite(s1, (channel & 0x02) ? HIGH : LOW);
  digitalWrite(s2, (channel & 0x04) ? HIGH : LOW);
  digitalWrite(s3, (channel & 0x08) ? HIGH : LOW);
  
  // Kurze Verzögerung für Multiplexer-Umschaltung
  delayMicroseconds(50);
}

int Multiplexer::readChannel(uint8_t channel) {
  if(channel > 15) return -1;
  
  // Wähle den Kanal aus
  selectChannel(channel);
  
  // Lese den analogen Wert
  int value = analogRead(analogPin);
  
  // Speichere den Wert und die Zeit
  lastValues[channel] = value;
  lastReadTime[channel] = millis();
  
  return value;
}

bool Multiplexer::hasChanged(uint8_t channel, int threshold) {
  if(channel > 15) return false;
  
  // Lese aktuellen Wert
  int currentValue = readChannel(channel);
  
  // Prüfe ob sich der Wert signifikant geändert hat
  if(lastValues[channel] == -1) {
    // Erster Aufruf - als Änderung werten
    return true;
  }
  
  return abs(currentValue - lastValues[channel]) >= threshold;
}
